@@ -4,12 +4,12 @@ adjustedProfileLik <- function(dispersion, y, design, offset, weights=NULL, adju
 # y is matrix: rows are genes/tags/transcripts, columns are samples/libraries
 # offset is matrix of the same dimensions as y
 # Yunshun Chen, Gordon Smyth, Aaron Lun
-# Created June 2010. Last modified 11 Sep 2014.
+# Created June 2010. Last modified 27 July 2015.
 {
 	if(any(dim(y)!=dim(offset))) offset <- expandAsMatrix(offset,dim(y))
 	ntags <- nrow(y)
 	nlibs <- ncol(y)
-	if(length(dispersion)==1) dispersion <- rep(dispersion,ntags)
+	dispersion <- expandAsMatrix(dispersion,dim(y),byrow=FALSE)
        
 #	Fit tagwise linear models. This is actually the most time-consuming
 #	operation that I can see for this function.
@@ -18,7 +18,7 @@ adjustedProfileLik <- function(dispersion, y, design, offset, weights=NULL, adju
 #	Compute log-likelihood.
 	mu <- fit$fitted
 	if(is.null(weights)) weights <- 1
-	if(dispersion[1] == 0){
+	if(all(dispersion == 0)){
 #		loglik <- rowSums(weights*dpois(y,lambda=mu,log = TRUE))
 		ll <- y*log(mu)-mu-lgamma(y+1)
 		ll[mu==0] <- 0
