@@ -4,8 +4,7 @@ glmQLFit <- function(y, ...)
 UseMethod("glmQLFit")
 
 glmQLFit.DGEList <- function(y, design=NULL, dispersion=NULL, offset=NULL, abundance.trend=TRUE, robust=FALSE, winsor.tail.p=c(0.05, 0.1), ...)
-#	Quasi-likelihood F-tests for DGE glms.
-# 	Yunshun Chen and Aaron Lun
+# 	Written by Yunshun Chen and Aaron Lun
 #	Created 05 November 2014.
 {
 	if(is.null(dispersion)) {
@@ -25,9 +24,9 @@ glmQLFit.DGEList <- function(y, design=NULL, dispersion=NULL, offset=NULL, abund
 	new("DGEGLM",fit)
 }
 
-glmQLFit.default <- function(y, design=NULL, dispersion=0.05, offset=NULL, lib.size=NULL, abundance.trend=TRUE, AveLogCPM=NULL, robust=FALSE, winsor.tail.p=c(0.05, 0.1), ...)
+glmQLFit.default <- function(y, design=NULL, dispersion=NULL, offset=NULL, lib.size=NULL, abundance.trend=TRUE, AveLogCPM=NULL, robust=FALSE, winsor.tail.p=c(0.05, 0.1), ...)
 # 	Fits a GLM and computes quasi-likelihood dispersions for each gene.
-# 	Yunshun Chen and Aaron Lun, based on code by Davis McCarthy and Gordon Smyth
+# 	Written by Yunshun Chen and Aaron Lun, based on code by Davis McCarthy and Gordon Smyth
 # 	Created 15 September 2014. Last modified 05 November 2014.
 {
 #	Check y
@@ -36,7 +35,7 @@ glmQLFit.default <- function(y, design=NULL, dispersion=0.05, offset=NULL, lib.s
 	nlib <- ncol(y)
 
 #	Check dispersion
-	if(is.null(dispersion)) dispersion <- 0.05
+	if(is.null(dispersion)) stop("No dispersion values provided.")
 
 #	Check offset and lib.size
 	if(is.null(offset)) {
@@ -109,7 +108,7 @@ glmQLFTest <- function(glmfit, coef=ncol(glmfit$design), contrast=NULL)
 
 plotQLDisp <- function(glmfit, xlab="Average Log2 CPM", ylab="Quarter-Root Mean Deviance", pch=16, cex=0.2, col.shrunk="red", col.trend="blue", col.raw="black", ...)
 # 	Plots the result of QL-based shrinkage.
-#	Aaron Lun, based on code by Davis McCarthy and Gordon Smyth
+#	written by Aaron Lun, based on code by Davis McCarthy and Gordon Smyth
 #	15 September 2014
 {
 	A <- glmfit$AveLogCPM
