@@ -9,7 +9,10 @@ nbinomDeviance <- function(y,mean,dispersion=0,weights=NULL)
 	if(!is.matrix(y)) y <- array(y, c(1L,length(y)), if(!is.null(names(y))) list(NULL,names(y)))
 
 	d <- nbinomUnitDeviance(y=y,mean=mean,dispersion=dispersion)
-	if(!is.null(weights)) d <- weights*d
+	if(!is.null(weights)) {
+		weights <- expandAsMatrix(weights, dim(d)) 
+		d <- weights*d
+	}
 	rowSums(d)
 }
 
@@ -33,7 +36,7 @@ nbinomUnitDeviance <- function(y,mean,dispersion=0)
 	if(lend < ntags) dispersion <- rep_len(dispersion, length.out=ntags)
 	if(lend > ntags && lend < nobs) dispersion <- rep_len(dispersion, length.out=nobs)
 
-	out <- .Call("R_compute_nbdev", y=y, mu=mean, phi=dispersion, PACKAGE="edgeR")
+	out <- .Call(.cR_compute_nbdev, y=y, mu=mean, phi=dispersion)
 
 #	Check error status
 	if (is.character(out)) stop(out)
